#!/bin/bash

# Errors are dependancies that are not installed
# depends, optional_depends, and requires are all supported
#
# Meta-spells may gnerate false FAILED messages
# Non-standard DEPENDS files may generate false FAILED messages
#  This includes exim.

. /etc/sorcery/config

function ok ()
{
	[[ $SILENCE_OK ]] && return 0
	message "$1${GREEN}OK${DEFAULT}"
}
function error ()
{
	message "$1${RED}FAILED${DEFAULT}"
}
function warning ()
{
	message "$1${YELLOW}WARNING${DEFAULT}"
}

function depends ()
{
	for i in $SPELLS ; do
		if [[ $i == $1 ]] ; then
			ok "\\t$1: "
			return 0
		fi
	done
	error "\\t$1: "
	return 1
}
function optional_depends ()
{
	for i in $SPELLS ; do
		if [[ $i == $1 ]] ; then
			ok "\\t$1: "
			return 0
		fi
	done
	
	# Only resort to using the DEPENDS_STATUS to see if an optional
	#  dependancy was us
	if grep -q "^$SPELL:$1:on:optional" $DEPENDS_STATUS ; then
		error "\\t$1: "
	else
	    ok "\\t$i: "
	fi
	return 0
}

#shamlessly stolen from subroutins
find_providers()  {

  for  PROVIDER  in  $(  find  $GRIMOIRE -follow  -name  provides  );  do
    if    grep  -q  $1  $PROVIDER
    then  basename  $( dirname  $PROVIDER  )
    fi
  done
}
#mostly stolen from subroutines too
function requires ()
{
	CANDIDATES=`find_providers $1`
    SATISFIED="false"

  for  CANDIDATE  in       $CANDIDATES;  do
    if    spell_installed  $CANDIDATE  ||
          in_depends       $CANDIDATE
    then  SATISFIED="true";  break
    fi
  done
  if ! $SATISFIED ; then
	  error "\\tRequires $1: "
  else
	  ok "\\tRequires $1: "
  fi
}

SILENCE_OK="yes"
[[ $1 == -v ]] && SILENCE_OK=""

SPELLS=`gaze installed | cut -d ':' -f1`

#Some spells' DEPENDS are just too weird for 
#  this simplistic script to handle, so we will remove them
BROKEN="cdrtools exim"
for i in $BROKEN ; do
	if echo "$SPELLS" | grep -q "^$i$" ; then
		echo "You have $i installed. This script cannot properly check this spell. "
		warning "Removing check of $i: "
		SPELLS=`echo "$SPELLS" | grep -v "^$i$"`
	fi
done

for SPELL in $SPELLS ; do
	echo "$SPELL:"
	DEP="$GRIMOIRE/`gaze where $SPELL`/$SPELL/DEPENDS"
	[ -n "$DEP" ] && [ -x "$DEP" ] && . $DEP
done
